%  demo_interact
%
%  cbm 2005-02-18

LEFTBTN = 1;
RIGHTBTN = 3;
USEOPENGL = 1;

disp('choose graphics rendering:');
disp('  0)  compatibility mode: disable OpenGL');
disp('  1)  autoselect (default)');
disp('  2)  force OpenGL');

USEOPENGL = input('? ');

if (USEOPENGL == 1)
  opengl autoselect
elseif (USEOPENGL == 2)
  % force OpenGL
  opengl autoselect
  set(GCF, 'Renderer', 'OpenGL');
elseif (USEOPENGL == 0)
  % let computer choose but NOT opengl
  opengl neverselect
  %set(GCF, 'Renderer', 'painters');
  %set(GCF, 'Renderer', 'ZBuffer');
else
  error('invalid selection');
end

load_image;

%  compute fourier coefficients (complex form)
imfour = fftshift(fft2(imdata));
%  real-valued by-products
imfcos = real(imfour);
imfsin = imag(imfour);
imfabs =  abs(imfour);
logimfabs = log(imfabs);
%  fourier mode numbers & mask matrices
k = [-floor(imsize(2)/2):ceil(imsize(2)/2)-1];
l = [-floor(imsize(1)/2):ceil(imsize(1)/2)-1];
[km lm] = meshgrid(k,l);
mask = zeros(size(km));


figure(1); clf;
colormap(copper);
figure(2); clf;
if (USEOPENGL == 0)   
  imagesc(k,l,B);
  grayT = [(1:50)/70]';
  grayTT = [grayT grayT grayT];
  grayT = [(25:.5:50)/50]';
  lgrayTT = [grayT grayT grayT];      
  mymap = [grayTT; lgrayTT];      
  colormap(mymap);
else
  colormap(jet);
end

RAD = 16;
disp('**************************************************************');
disp('draw a mask using the mouse');
disp('left mouse btn to draw, right mouse button to erase');
disp('press ''c'' to clear the mask');
disp('press 1-6 to change brush size');
disp('press i to invert the mask');
disp('press s to save, l to load the mask');
disp('press q to quit');

dirty = 1;

while(1)  % endless loop
  if (dirty)
    figure(1); %clf;
    impart = ifft2(ifftshift(mask.*imfour));
    imagesc(k,l,real(impart));
    title(['partial image for ' file])
    xlabel('x');
    ylabel('y');
    axis image;
    
    figure(2);
    if (USEOPENGL == 0)
      maxA = max(max(logimfabs));
      minA = min(min(logimfabs));
      B = (logimfabs+maxA).*(mask==0) + logimfabs.*(mask==1);
      B(1,1) = minA;
      B(2,1) = maxA;
      B(3,1) = 2*maxA;
      imagesc(k,l,B);
      grayT = [(1:50)/70]';
      grayTT = [grayT grayT grayT];
      grayT = [(25:.5:50)/50]';
      lgrayTT = [grayT grayT grayT];      
      mymap = [grayTT; lgrayTT];      
      colormap(mymap);

    else
      imagesc(k,l,logimfabs);
      % the visualization alpha mask lets all coefficients show
      % through but the 1's in the mask are "brighter"
      visalphamask = .2*(~mask) + mask;
      alpha(visalphamask);
    end
    
    title('fourier coefficients and mask');
    xlabel('k');
    ylabel('l');
    axis image;
    dirty = 0;
  end

  [K,L,B] = ginput(1);
  % detect and trap enter key
  if isempty(B)
    B = -1;  % some unused case below
  end
  switch(B)
   case LEFTBTN
    kc = round(K);   lc = round(L);
    mask = mask | ((km-kc).^2 + (lm-lc).^2 <= RAD^2);
    dirty = 1;
   case RIGHTBTN
    kc = round(K);   lc = round(L);
    mask = mask & ~((km-kc).^2 + (lm-lc).^2 <= RAD^2);
    dirty = 1;
   case 99  % c key
    mask = zeros(size(mask));
    dirty = 1;
   case 113 % q key
    break;
   case 115  % s kay
    save mask
   case 108  % l key
    load mask
    dirty = 1;
   case 105  % i key
    mask = ~mask;
    dirty = 1;
   case 49  % 1 key
    RAD = 2;
    disp(['brush set to radius ' num2str(RAD)]);
   case 50  % 2 key
    RAD = 4;
    disp(['brush set to radius ' num2str(RAD)]);
   case 51  % 3 key
    RAD = 8;
    disp(['brush set to radius ' num2str(RAD)]);
   case 52  % 4 key
    RAD = 16;
    disp(['brush set to radius ' num2str(RAD)]);
   case 53  % 5 key
    RAD = 32;
    disp(['brush set to radius ' num2str(RAD)]);
   case 54  % 6 key
    RAD = 64;
    disp(['brush set to radius ' num2str(RAD)]);
  end
end

%  count number of nonzero fourier modes.
keep = sum(mask(:));
disp(['number of nonzero fourier modes = ' num2str(keep)])
disp(['percent information = ' num2str(100*keep/imsize(1)/imsize(2))])
